#include "buildingeditordialog.h"
#include "ui_buildingeditordialog.h"
#include <QMessageBox>
#include <QSqlRecord>
#include <QSqlError>

BuildingEditorDialog::BuildingEditorDialog(QWidget *parent) :
    QDialog(parent),
    _ui(new Ui::BuildingEditorDialog)
{
    _ui->setupUi(this);

    setupModel();
    setupUi();
    setupMapper();
}

BuildingEditorDialog::~BuildingEditorDialog()
{
    delete _ui;
}

void BuildingEditorDialog::addButton_Clicked()
{
    _model->insertRow(_model->rowCount());
    _mapper->toLast();
}

void BuildingEditorDialog::removeButton_Clicked()
{
    _model->removeRow(_mapper->currentIndex());
    _mapper->toFirst();
}

void BuildingEditorDialog::submitButton_Clicked()
{
    _model->database().transaction();

    if (_model->submitAll())
    {
        _model->database().commit();
        _model->select();
        _mapper->toFirst();
    } else
    {
        _model->database().rollback();
        QMessageBox::warning(this, trUtf8("Hiba történt a mentéskor!"), trUtf8("Az adatbázis a kvetkező hibát jelezte: %1").arg(_model->lastError().text()));
    }
}

void BuildingEditorDialog::model_BeforeInsert(QSqlRecord& record)
{
    if (record.value("comment").isNull()) // ha a komment oszlop üres
        record.setValue("comment", trUtf8("új hírdetés"));
}

void BuildingEditorDialog::updateButtons(int row)
{
    _ui->prevButton->setEnabled(row > 0);
    _ui->nextButton->setEnabled(row < _model->rowCount() - 1);
}


void BuildingEditorDialog::setupModel()
{
    _model = new QSqlRelationalTableModel(this);
    _model->setTable("building");
    _model->setSort(1, Qt::AscendingOrder);
    _model->setEditStrategy(QSqlTableModel::OnManualSubmit);
    _model->setRelation(2, QSqlRelation("city", "id", "name"));
    _model->select();
}

void BuildingEditorDialog::setupUi()
{
    QSqlTableModel* cityModel = new QSqlTableModel(this); //városok modellje
    cityModel->setTable("city");
    cityModel->select();

    _ui->cityComboBox->setModel(cityModel); // modell beállítása a legörgülő menüre
    _ui->cityComboBox->setModelColumn(1); // modell oszlopának beállítása

    _ui->shoreTypeComboBox->addItem(trUtf8("Homokos"));
    _ui->shoreTypeComboBox->addItem(trUtf8("Sziklás"));
    _ui->shoreTypeComboBox->addItem(trUtf8("Kavicsos"));
    _ui->shoreTypeComboBox->addItem(trUtf8("Apró kavicsos"));

    connect(_ui->addButton, SIGNAL(clicked()), this, SLOT(addButton_Clicked()));
    connect(_ui->removeButton, SIGNAL(clicked()), this, SLOT(removeButton_Clicked()));
    connect(_ui->submitButton, SIGNAL(clicked()), this, SLOT(submitButton_Clicked()));
    connect(_ui->revertButton, SIGNAL(clicked()), _model, SLOT(revertAll()));

    setWindowTitle(trUtf8("Épületek szerkesztése"));
    setFixedSize(650, 280);
}

void BuildingEditorDialog::setupMapper()
{
    _mapper = new QDataWidgetMapper(this); // vezérlő adattásítása
    _mapper->setModel(_model);
    _mapper->addMapping(_ui->nameLineEdit, 1); // vezérlő társítása az oszlophoz
    _mapper->addMapping(_ui->cityComboBox, 2);
    _mapper->addMapping(_ui->streetLineEdit, 3);
    _mapper->addMapping(_ui->seaDistanceSpinBox, 4);
    _mapper->addMapping(_ui->shoreTypeComboBox, 5, "currentIndex"); // társított tulajdonság beállítása
    _mapper->addMapping(_ui->featureListWidget, 6, "features");
    _mapper->addMapping(_ui->commentTextBox, 7);

    connect(_ui->prevButton, SIGNAL(clicked()), _mapper, SLOT(toPrevious())); // előző elemre lépés
    connect(_ui->nextButton, SIGNAL(clicked()), _mapper, SLOT(toNext())); // követlező elemre lépés
    connect(_mapper, SIGNAL(currentIndexChanged(int)), this, SLOT(updateButtons(int)));

    _mapper->toFirst(); // első elemre lépés
}
